@@ -39,6 +39,7 @@ MAX_ACCOUNT_NAME_LENGTH = 256
 MAX_CONTAINER_NAME_LENGTH = 256
 VALID_API_VERSIONS = ["v1", "v1.0"]
 EXTRA_HEADER_COUNT = 0
+AUTO_CREATE_ACCOUNT_PREFIX = '.'
 
 # If adding an entry to DEFAULT_CONSTRAINTS, note that
 # these constraints are automatically published by the
@@ -58,6 +59,7 @@ DEFAULT_CONSTRAINTS = {
     'max_container_name_length': MAX_CONTAINER_NAME_LENGTH,
     'valid_api_versions': VALID_API_VERSIONS,
     'extra_header_count': EXTRA_HEADER_COUNT,
+    'auto_create_account_prefix': AUTO_CREATE_ACCOUNT_PREFIX,
 }
 
 SWIFT_CONSTRAINTS_LOADED = False
@@ -76,7 +78,7 @@ def reload_constraints():
     constraints_conf = ConfigParser()
     if constraints_conf.read(utils.SWIFT_CONF_FILE):
         SWIFT_CONSTRAINTS_LOADED = True
-        for name in DEFAULT_CONSTRAINTS:
+        for name, default in DEFAULT_CONSTRAINTS.items():
             try:
                 value = constraints_conf.get('swift-constraints', name)
             except NoOptionError:
@@ -85,9 +87,12 @@ def reload_constraints():
                 # We are never going to find the section for another option
                 break
             else:
-                try:
-                    value = int(value)
-                except ValueError:
+                if isinstance(default, int):
+                    value = int(value)  # Go ahead and let it error
+                elif isinstance(default, str):
+                    pass  # No translation needed, I guess
+                else:
+                    # Hope we want a list!
                     value = utils.list_from_csv(value)
                 OVERRIDE_CONSTRAINTS[name] = value
     for name, default in DEFAULT_CONSTRAINTS.items():
@@ -100,10 +105,6 @@ def reload_constraints():
 reload_constraints()
 
 
-# Maximum slo segments in buffer
-MAX_BUFFERED_SLO_SEGMENTS = 10000
-
-
 # By default the maximum number of allowed headers depends on the number of max
 # allowed metadata settings plus a default value of 36 for swift internally
 # generated headers and regular http headers.  If for some reason this is not
@@ -139,12 +140,12 @@ def check_metadata(req, target_type):
             continue
         key = key[len(prefix):]
         if not key:
-            return HTTPBadRequest(body=b'Metadata name cannot be empty',
+            return HTTPBadRequest(body='Metadata name cannot be empty',
                                   request=req, content_type='text/plain')
         bad_key = not check_utf8(wsgi_to_str(key))
         bad_value = value and not check_utf8(wsgi_to_str(value))
         if target_type in ('account', 'container') and (bad_key or bad_value):
-            return HTTPBadRequest(body=b'Metadata must be valid UTF-8',
+            return HTTPBadRequest(body='Metadata must be valid UTF-8',
                                   request=req, content_type='text/plain')
         meta_count += 1
         meta_size += len(key) + len(value)
@@ -160,11 +161,11 @@ def check_metadata(req, target_type):
                 request=req, content_type='text/plain')
         if meta_count > MAX_META_COUNT:
             return HTTPBadRequest(
-                body=b'Too many metadata items; max %d' % MAX_META_COUNT,
+                body='Too many metadata items; max %d' % MAX_META_COUNT,
                 request=req, content_type='text/plain')
         if meta_size > MAX_META_OVERALL_SIZE:
             return HTTPBadRequest(
-                body=b'Total metadata too large; max %d'
+                body='Total metadata too large; max %d'
                 % MAX_META_OVERALL_SIZE,
                 request=req, content_type='text/plain')
     return None
@@ -187,22 +188,22 @@ def check_object_creation(req, object_name):
         ml = req.message_length()
     except ValueError as e:
         return HTTPBadRequest(request=req, content_type='text/plain',
-                              body=str(e).encode('ascii'))
+                              body=str(e))
     except AttributeError as e:
         return HTTPNotImplemented(request=req, content_type='text/plain',
-                                  body=str(e).encode('ascii'))
+                                  body=str(e))
     if ml is not None and ml > MAX_FILE_SIZE:
-        return HTTPRequestEntityTooLarge(body=b'Your request is too large.',
+        return HTTPRequestEntityTooLarge(body='Your request is too large.',
                                          request=req,
                                          content_type='text/plain')
     if req.content_length is None and \
             req.headers.get('transfer-encoding') != 'chunked':
-        return HTTPLengthRequired(body=b'Missing Content-Length header.',
+        return HTTPLengthRequired(body='Missing Content-Length header.',
                                   request=req,
                                   content_type='text/plain')
 
     if len(object_name) > MAX_OBJECT_NAME_LENGTH:
-        return HTTPBadRequest(body=b'Object name length of %d longer than %d' %
+        return HTTPBadRequest(body='Object name length of %d longer than %d' %
                               (len(object_name), MAX_OBJECT_NAME_LENGTH),
                               request=req, content_type='text/plain')
 
@@ -217,7 +218,7 @@ def check_object_creation(req, object_name):
                               content_type='text/plain')
 
     if not check_utf8(wsgi_to_str(req.headers['Content-Type'])):
-        return HTTPBadRequest(request=req, body=b'Invalid Content-Type',
+        return HTTPBadRequest(request=req, body='Invalid Content-Type',
                               content_type='text/plain')
     return check_metadata(req, 'object')
 
@@ -300,7 +301,7 @@ def valid_timestamp(request):
     try:
         return request.timestamp
     except exceptions.InvalidTimestamp as e:
-        raise HTTPBadRequest(body=str(e).encode('ascii'), request=request,
+        raise HTTPBadRequest(body=str(e), request=request,
                              content_type='text/plain')
 
 
@@ -325,13 +326,13 @@ def check_delete_headers(request):
         except ValueError:
             raise HTTPBadRequest(request=request,
                                  content_type='text/plain',
-                                 body=b'Non-integer X-Delete-After')
+                                 body='Non-integer X-Delete-After')
         actual_del_time = utils.normalize_delete_at_timestamp(
             now + x_delete_after)
         if int(actual_del_time) <= now:
             raise HTTPBadRequest(request=request,
                                  content_type='text/plain',
-                                 body=b'X-Delete-After in past')
+                                 body='X-Delete-After in past')
         request.headers['x-delete-at'] = actual_del_time
         del request.headers['x-delete-after']
 
@@ -341,21 +342,22 @@ def check_delete_headers(request):
                 int(request.headers['x-delete-at'])))
         except ValueError:
             raise HTTPBadRequest(request=request, content_type='text/plain',
-                                 body=b'Non-integer X-Delete-At')
+                                 body='Non-integer X-Delete-At')
 
         if x_delete_at <= now and not utils.config_true_value(
                 request.headers.get('x-backend-replication', 'f')):
             raise HTTPBadRequest(request=request, content_type='text/plain',
-                                 body=b'X-Delete-At in past')
+                                 body='X-Delete-At in past')
     return request
 
 
-def check_utf8(string):
+def check_utf8(string, internal=False):
     """
     Validate if a string is valid UTF-8 str or unicode and that it
-    does not contain any null character.
+    does not contain any reserved characters.
 
     :param string: string to be validated
+    :param internal: boolean, allows reserved characters if True
     :returns: True if the string is valid utf-8 str or unicode and
               contains no null characters, False otherwise
     """
@@ -386,7 +388,9 @@ def check_utf8(string):
         if any(0xD800 <= ord(codepoint) <= 0xDFFF
                for codepoint in decoded):
             return False
-        return b'\x00' not in encoded
+        if b'\x00' != utils.RESERVED_BYTE and b'\x00' in encoded:
+            return False
+        return True if internal else utils.RESERVED_BYTE not in encoded
     # If string is unicode, decode() will raise UnicodeEncodeError
     # So, we should catch both UnicodeDecodeError & UnicodeEncodeError
     except UnicodeError:
@@ -407,19 +411,21 @@ def check_name_format(req, name, target_type):
     if not name:
         raise HTTPPreconditionFailed(
             request=req,
-            body=b'%s name cannot be empty' % target_type)
-    if isinstance(name, six.text_type):
-        name = name.encode('utf-8')
-    if b'/' in name:
+            body='%s name cannot be empty' % target_type)
+    if six.PY2:
+        if isinstance(name, six.text_type):
+            name = name.encode('utf-8')
+    if '/' in name:
         raise HTTPPreconditionFailed(
             request=req,
-            body=b'%s name cannot contain slashes' % target_type)
+            body='%s name cannot contain slashes' % target_type)
     return name
 
+
 check_account_format = functools.partial(check_name_format,
-                                         target_type=b'Account')
+                                         target_type='Account')
 check_container_format = functools.partial(check_name_format,
-                                           target_type=b'Container')
+                                           target_type='Container')
 
 
 def valid_api_version(version):
