@@ -123,16 +123,20 @@ Example usage of this middleware via ``swift``:
 """
 
 
-import cgi
 import json
+import six
 import time
 
+from six.moves.urllib.parse import urlparse
+
+from swift.common.request_helpers import html_escape
 from swift.common.utils import human_readable, split_path, config_true_value, \
-    quote, register_swift_info, get_logger, urlparse
+    quote, get_logger
+from swift.common.registry import register_swift_info
 from swift.common.wsgi import make_env, WSGIContext
 from swift.common.http import is_success, is_redirection, HTTP_NOT_FOUND
 from swift.common.swob import Response, HTTPMovedPermanently, HTTPNotFound, \
-    Request
+    Request, wsgi_quote, wsgi_to_str, str_to_wsgi
 from swift.proxy.controllers.base import get_container_info
 
 
@@ -145,6 +149,12 @@ class _StaticWebContext(WSGIContext):
     that might need to be handled to make keeping contextual
     information about the request a bit simpler than storing it in
     the WSGI env.
+
+    :param staticweb: The staticweb middleware object in use.
+    :param version: A WSGI string representation of the swift api version.
+    :param account: A WSGI string representation of the account name.
+    :param container: A WSGI string representation of the container name.
+    :param obj: A WSGI string representation of the object name.
     """
 
     def __init__(self, staticweb, version, account, container, obj):
@@ -199,7 +209,7 @@ class _StaticWebContext(WSGIContext):
         self._error, self._listings, self._listings_css and self._dir_type.
 
         :param env: The WSGI environment dict.
-        :return container_info: The container_info dict.
+        :return: The container_info dict.
         """
         self._index = self._error = self._listings = self._listings_css = \
             self._dir_type = None
@@ -221,11 +231,11 @@ class _StaticWebContext(WSGIContext):
 
         :param env: The original WSGI environment dict.
         :param start_response: The original WSGI start_response hook.
-        :param prefix: Any prefix desired for the container listing.
+        :param prefix: Any WSGI-str prefix desired for the container listing.
         """
-        label = env['PATH_INFO']
+        label = wsgi_to_str(env['PATH_INFO'])
         if self._listings_label:
-            groups = env['PATH_INFO'].split('/')
+            groups = wsgi_to_str(env['PATH_INFO']).split('/')
             label = '{0}/{1}'.format(self._listings_label,
                                      '/'.join(groups[4:]))
 
@@ -234,7 +244,7 @@ class _StaticWebContext(WSGIContext):
                 'Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' \
                 '<html>\n' \
                 '<head>\n' \
-                '<title>Listing of %s</title>\n' % cgi.escape(label)
+                '<title>Listing of %s</title>\n' % html_escape(label)
             if self._listings_css:
                 body += '  <link rel="stylesheet" type="text/css" ' \
                     'href="%s" />\n' % self._build_css_path(prefix or '')
@@ -260,19 +270,19 @@ class _StaticWebContext(WSGIContext):
             env, 'GET', '/%s/%s/%s' % (
                 self.version, self.account, self.container),
             self.agent, swift_source='SW')
-        tmp_env['QUERY_STRING'] = 'delimiter=/&format=json'
+        tmp_env['QUERY_STRING'] = 'delimiter=/'
         if prefix:
-            tmp_env['QUERY_STRING'] += '&prefix=%s' % quote(prefix)
+            tmp_env['QUERY_STRING'] += '&prefix=%s' % wsgi_quote(prefix)
         else:
             prefix = ''
         resp = self._app_call(tmp_env)
         if not is_success(self._get_status_int()):
             return self._error_response(resp, env, start_response)
         listing = None
-        body = ''.join(resp)
+        body = b''.join(resp)
         if body:
             listing = json.loads(body)
-        if not listing:
+        if prefix and not listing:
             resp = HTTPNotFound()(env, self._start_response)
             return self._error_response(resp, env, start_response)
         headers = {'Content-Type': 'text/html; charset=UTF-8'}
@@ -280,7 +290,8 @@ class _StaticWebContext(WSGIContext):
                'Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">\n' \
                '<html>\n' \
                ' <head>\n' \
-               '  <title>Listing of %s</title>\n' % cgi.escape(label)
+               '  <title>Listing of %s</title>\n' % \
+               html_escape(label)
         if self._listings_css:
             body += '  <link rel="stylesheet" type="text/css" ' \
                     'href="%s" />\n' % (self._build_css_path(prefix))
@@ -299,7 +310,7 @@ class _StaticWebContext(WSGIContext):
                 '    <th class="colname">Name</th>\n' \
                 '    <th class="colsize">Size</th>\n' \
                 '    <th class="coldate">Date</th>\n' \
-                '   </tr>\n' % cgi.escape(label)
+                '   </tr>\n' % html_escape(label)
         if prefix:
             body += '   <tr id="parent" class="item">\n' \
                     '    <td class="colname"><a href="../">../</a></td>\n' \
@@ -308,33 +319,37 @@ class _StaticWebContext(WSGIContext):
                     '   </tr>\n'
         for item in listing:
             if 'subdir' in item:
-                subdir = item['subdir'].encode("utf-8")
+                subdir = item['subdir'] if six.PY3 else  \
+                    item['subdir'].encode('utf-8')
                 if prefix:
-                    subdir = subdir[len(prefix):]
+                    subdir = subdir[len(wsgi_to_str(prefix)):]
                 body += '   <tr class="item subdir">\n' \
                         '    <td class="colname"><a href="%s">%s</a></td>\n' \
                         '    <td class="colsize">&nbsp;</td>\n' \
                         '    <td class="coldate">&nbsp;</td>\n' \
                         '   </tr>\n' % \
-                        (quote(subdir), cgi.escape(subdir))
+                        (quote(subdir), html_escape(subdir))
         for item in listing:
             if 'name' in item:
-                name = item['name'].encode("utf-8")
+                name = item['name'] if six.PY3 else  \
+                    item['name'].encode('utf-8')
                 if prefix:
-                    name = name[len(prefix):]
-                content_type = item['content_type'].encode("utf-8")
+                    name = name[len(wsgi_to_str(prefix)):]
+                content_type = item['content_type'] if six.PY3 else  \
+                    item['content_type'].encode('utf-8')
                 bytes = human_readable(item['bytes'])
                 last_modified = (
-                    cgi.escape(item['last_modified'].encode("utf-8")).
+                    html_escape(item['last_modified'] if six.PY3 else
+                                item['last_modified'].encode('utf-8')).
                     split('.')[0].replace('T', ' '))
                 body += '   <tr class="item %s">\n' \
                         '    <td class="colname"><a href="%s">%s</a></td>\n' \
                         '    <td class="colsize">%s</td>\n' \
                         '    <td class="coldate">%s</td>\n' \
                         '   </tr>\n' % \
-                        (' '.join('type-' + cgi.escape(t.lower(), quote=True)
+                        (' '.join('type-' + html_escape(t.lower())
                                   for t in content_type.split('/')),
-                         quote(name), cgi.escape(name),
+                         quote(name), html_escape(name),
                          bytes, last_modified)
         body += '  </table>\n' \
                 ' </body>\n' \
@@ -362,7 +377,8 @@ class _StaticWebContext(WSGIContext):
             env['wsgi.url_scheme'] = self.url_scheme
         if self.url_host:
             env['HTTP_HOST'] = self.url_host
-        resp = HTTPMovedPermanently(location=(env['PATH_INFO'] + '/'))
+        resp = HTTPMovedPermanently(
+            location=wsgi_quote(env['PATH_INFO'] + '/'))
         return resp(env, start_response)
 
     def handle_container(self, env, start_response):
@@ -393,7 +409,7 @@ class _StaticWebContext(WSGIContext):
         tmp_env['HTTP_USER_AGENT'] = \
             '%s StaticWeb' % env.get('HTTP_USER_AGENT')
         tmp_env['swift.source'] = 'SW'
-        tmp_env['PATH_INFO'] += self._index
+        tmp_env['PATH_INFO'] += str_to_wsgi(self._index)
         resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
         if status_int == HTTP_NOT_FOUND:
@@ -450,7 +466,7 @@ class _StaticWebContext(WSGIContext):
             tmp_env['swift.source'] = 'SW'
             if not tmp_env['PATH_INFO'].endswith('/'):
                 tmp_env['PATH_INFO'] += '/'
-            tmp_env['PATH_INFO'] += self._index
+            tmp_env['PATH_INFO'] += str_to_wsgi(self._index)
             resp = self._app_call(tmp_env)
             status_int = self._get_status_int()
             if is_success(status_int) or is_redirection(status_int):
@@ -465,10 +481,10 @@ class _StaticWebContext(WSGIContext):
                     env, 'GET', '/%s/%s/%s' % (
                         self.version, self.account, self.container),
                     self.agent, swift_source='SW')
-                tmp_env['QUERY_STRING'] = 'limit=1&format=json&delimiter' \
-                    '=/&limit=1&prefix=%s' % quote(self.obj + '/')
+                tmp_env['QUERY_STRING'] = 'limit=1&delimiter=/&prefix=%s' % (
+                    quote(wsgi_to_str(self.obj) + '/'), )
                 resp = self._app_call(tmp_env)
-                body = ''.join(resp)
+                body = b''.join(resp)
                 if not is_success(self._get_status_int()) or not body or \
                         not json.loads(body):
                     resp = HTTPNotFound()(env, self._start_response)
