@@ -44,7 +44,8 @@ from swift.common.utils import set_swift_dir
 
 from test import get_config, listen_zero
 
-from test.unit import debug_logger, FakeMemcache
+from test.debug_logger import debug_logger
+from test.unit import FakeMemcache
 # importing skip_if_no_xattrs so that functional tests can grab it from the
 # test.functional namespace. Importing SkipTest so this works under both
 # nose and testr test runners.
@@ -581,16 +582,11 @@ def in_process_setup(the_object_server=object_server):
         'swift_dir': _testdir,
         'mount_check': 'false',
         'client_timeout': '4',
+        'container_update_timeout': '3',
         'allow_account_management': 'true',
         'account_autocreate': 'true',
         'allow_versions': 'True',
         'allow_versioned_writes': 'True',
-        # TODO: move this into s3api config loader because they are
-        #       required by only s3api
-        'allowed_headers':
-            "Content-Disposition, Content-Encoding, X-Delete-At, "
-            "X-Object-Manifest, X-Static-Large-Object, Cache-Control, "
-            "Content-Language, Expires, X-Robots-Tag",
         # Below are values used by the functional test framework, as well as
         # by the various in-process swift servers
         'auth_uri': 'http://127.0.0.1:%d/auth/v1.0/' % prolis.getsockname()[1],
@@ -652,9 +648,6 @@ def in_process_setup(the_object_server=object_server):
                       'port': con2lis.getsockname()[1]}], 30),
                     f)
 
-    # Default to only 4 seconds for in-process functional test runs
-    eventlet.wsgi.WRITE_TIMEOUT = 4
-
     def get_logger_name(name):
         if show_debug_logs:
             return debug_logger(name)
