@@ -1,4 +1,5 @@
-# Copyright (c) 2015 OpenStack Foundation
+# -*- coding: utf-8 -*-
+# Copyright (c) 2015-2021 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,16 +22,17 @@ import boto
 # pylint: disable-msg=E0611,F0401
 from distutils.version import StrictVersion
 
+import calendar
 import email.parser
 from email.utils import formatdate, parsedate
 from time import mktime
-from hashlib import md5
 import six
-from six.moves.urllib.parse import quote
 
 import test.functional as tf
 
 from swift.common.middleware.s3api.etree import fromstring
+from swift.common.middleware.s3api.utils import S3Timestamp
+from swift.common.utils import md5, quote
 
 from test.functional.s3api import S3ApiBase
 from test.functional.s3api.s3_test_client import Connection
@@ -59,9 +61,9 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers, etag)
 
     def test_object(self):
-        obj = 'object name with %-sign'
+        obj = u'object name with %-sign 🙂'
         content = b'abc123'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
 
         # PUT Object
         status, headers, body = \
@@ -98,21 +100,32 @@ class TestS3ApiObject(S3ApiBase):
 
         elem = fromstring(body, 'CopyObjectResult')
         self.assertTrue(elem.find('LastModified').text is not None)
-        last_modified_xml = elem.find('LastModified').text
+        copy_resp_last_modified_xml = elem.find('LastModified').text
         self.assertTrue(elem.find('ETag').text is not None)
         self.assertEqual(etag, elem.find('ETag').text.strip('"'))
         self._assertObjectEtag(dst_bucket, dst_obj, etag)
 
-        # Check timestamp on Copy:
+        # Check timestamp on Copy in listing:
         status, headers, body = \
             self.conn.make_request('GET', dst_bucket)
         self.assertEqual(status, 200)
         elem = fromstring(body, 'ListBucketResult')
-
-        # FIXME: COPY result drops milli/microseconds but GET doesn't
         self.assertEqual(
-            elem.find('Contents').find("LastModified").text.rsplit('.', 1)[0],
-            last_modified_xml.rsplit('.', 1)[0])
+            elem.find('Contents').find("LastModified").text,
+            copy_resp_last_modified_xml)
+
+        # GET Object copy
+        status, headers, body = \
+            self.conn.make_request('GET', dst_bucket, dst_obj)
+        self.assertEqual(status, 200)
+
+        self.assertCommonResponseHeaders(headers, etag)
+        self.assertTrue(headers['last-modified'] is not None)
+        self.assertEqual(
+            float(S3Timestamp.from_s3xmlformat(copy_resp_last_modified_xml)),
+            calendar.timegm(parsedate(headers['last-modified'])))
+        self.assertTrue(headers['content-type'] is not None)
+        self.assertEqual(headers['content-length'], str(len(content)))
 
         # GET Object
         status, headers, body = \
@@ -158,6 +171,13 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(get_error_code(body), 'NoSuchBucket')
         self.assertEqual(headers['content-type'], 'application/xml')
 
+    def test_put_object_name_too_long(self):
+        status, headers, body = self.conn.make_request(
+            'PUT', self.bucket,
+            'x' * (tf.cluster_info['swift']['max_object_name_length'] + 1))
+        self.assertEqual(get_error_code(body), 'KeyTooLongError')
+        self.assertEqual(headers['content-type'], 'application/xml')
+
     def test_put_object_copy_error(self):
         obj = 'object'
         self.conn.make_request('PUT', self.bucket, obj)
@@ -252,7 +272,7 @@ class TestS3ApiObject(S3ApiBase):
 
     def test_put_object_content_encoding(self):
         obj = 'object'
-        etag = md5().hexdigest()
+        etag = md5(usedforsecurity=False).hexdigest()
         headers = {'Content-Encoding': 'gzip'}
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj, headers)
@@ -267,7 +287,7 @@ class TestS3ApiObject(S3ApiBase):
     def test_put_object_content_md5(self):
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         headers = {'Content-MD5': calculate_md5(content)}
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj, headers, content)
@@ -278,7 +298,7 @@ class TestS3ApiObject(S3ApiBase):
     def test_put_object_content_type(self):
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         headers = {'Content-Type': 'text/plain'}
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj, headers, content)
@@ -320,7 +340,7 @@ class TestS3ApiObject(S3ApiBase):
     def test_put_object_expect(self):
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         headers = {'Expect': '100-continue'}
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj, headers, content)
@@ -333,7 +353,7 @@ class TestS3ApiObject(S3ApiBase):
             expected_headers = req_headers
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj,
                                    req_headers, content)
@@ -389,7 +409,7 @@ class TestS3ApiObject(S3ApiBase):
     def test_put_object_storage_class(self):
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         headers = {'X-Amz-Storage-Class': 'STANDARD'}
         status, headers, body = \
             self.conn.make_request('PUT', self.bucket, obj, headers, content)
@@ -435,7 +455,7 @@ class TestS3ApiObject(S3ApiBase):
     def test_put_object_copy_source(self):
         obj = 'object'
         content = b'abcdefghij'
-        etag = md5(content).hexdigest()
+        etag = md5(content, usedforsecurity=False).hexdigest()
         self.conn.make_request('PUT', self.bucket, obj, body=content)
 
         dst_bucket = 'dst-bucket'
@@ -521,7 +541,7 @@ class TestS3ApiObject(S3ApiBase):
         obj = 'object'
         dst_bucket = 'dst-bucket'
         dst_obj = 'dst_object'
-        etag = md5().hexdigest()
+        etag = md5(usedforsecurity=False).hexdigest()
         self.conn.make_request('PUT', self.bucket, obj)
         self.conn.make_request('PUT', dst_bucket)
 
@@ -541,7 +561,7 @@ class TestS3ApiObject(S3ApiBase):
         obj = 'object'
         dst_bucket = 'dst-bucket'
         dst_obj = 'dst_object'
-        etag = md5().hexdigest()
+        etag = md5(usedforsecurity=False).hexdigest()
         self.conn.make_request('PUT', self.bucket, obj)
         self.conn.make_request('PUT', dst_bucket)
 
@@ -561,7 +581,7 @@ class TestS3ApiObject(S3ApiBase):
         obj = 'object'
         dst_bucket = 'dst-bucket'
         dst_obj = 'dst_object'
-        etag = md5().hexdigest()
+        etag = md5(usedforsecurity=False).hexdigest()
         self.conn.make_request('PUT', self.bucket, obj)
         self.conn.make_request('PUT', dst_bucket)
 
@@ -580,7 +600,7 @@ class TestS3ApiObject(S3ApiBase):
         obj = 'object'
         dst_bucket = 'dst-bucket'
         dst_obj = 'dst_object'
-        etag = md5().hexdigest()
+        etag = md5(usedforsecurity=False).hexdigest()
         self.conn.make_request('PUT', self.bucket, obj)
         self.conn.make_request('PUT', dst_bucket)
 
@@ -763,6 +783,28 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
 
+        # check we can use the last modified time from the listing...
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket)
+        elem = fromstring(body, 'ListBucketResult')
+        last_modified = elem.find('./Contents/LastModified').text
+        listing_datetime = S3Timestamp.from_s3xmlformat(last_modified)
+        # Make sure there's no fractions of a second
+        self.assertEqual(int(listing_datetime), float(listing_datetime))
+        header_datetime = formatdate(int(listing_datetime))
+
+        headers = {'If-Unmodified-Since': header_datetime}
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket, obj, headers=headers)
+        self.assertEqual(status, 200)
+        self.assertCommonResponseHeaders(headers)
+
+        headers = {'If-Modified-Since': header_datetime}
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket, obj, headers=headers)
+        self.assertEqual(status, 304)
+        self.assertCommonResponseHeaders(headers)
+
     def test_get_object_if_match(self):
         obj = 'object'
         self.conn.make_request('PUT', self.bucket, obj)
